import argparse
import os
import sys

sys.path.append(os.path.dirname(__file__))  # noqa

from check_release import check_release
from prepare_release import prepare_release
from commit_release import git_commit_and_push
from create_github_release import create_github_release
from deploy_release import deploy_release


def check(arguments):
    check_release()


def prepare(arguments):
    prepare_release()


def commit(arguments):
    git_commit_and_push()


def githubrelease(arguments):
    create_github_release()


def deploy(arguments):
    deploy_release()


def parse_arguments():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()

    check_parser = subparsers.add_parser("check")
    check_parser.set_defaults(func=check)

    prepare_parser = subparsers.add_parser("prepare")
    prepare_parser.set_defaults(func=prepare)

    commit_parser = subparsers.add_parser("commit")
    commit_parser.set_defaults(func=commit)

    githubrelease_parser = subparsers.add_parser("githubrelease")
    githubrelease_parser.set_defaults(func=githubrelease)

    deploy_parser = subparsers.add_parser("deploy")
    deploy_parser.set_defaults(func=deploy)

    arguments = parser.parse_args()

    return arguments


def main():
    arguments = parse_arguments()
    arguments.func(arguments)



import os
import sys

sys.path.append(os.path.dirname(__file__))  # noqa

from datetime import datetime

from base import CI_SYSTEM, RELEASE_FILE, run_process


def check_release():
    if not os.path.exists(RELEASE_FILE):
        print("Not releasing a new version because there is no RELEASE file.")
        if CI_SYSTEM == "circleci":
            run_process(["circleci", "step", "halt"])
    elif CI_SYSTEM == "travis":
        print("true")



import os
import re
import sys

sys.path.append(os.path.dirname(__file__))  # noqa

from datetime import datetime

from base import (
    run_process,
    get_release_info,
    CHANGELOG_FILE,
    CHANGELOG_HEADER,
    ROOT,
    VERSION_HEADER,
    VERSION_STRINGS,
)


def update_version_strings(file_path, new_version):
    VERSION_REGEX = re.compile(
        r"(^(__)?version(__)?\s*=\s*\")(?P<version>\d+\.\d+\.\d+)\"$"
    )
    with open(file_path, "r+") as f:
        content = f.read()
        f.seek(0)
        f.write(
            re.sub(
                VERSION_REGEX,
                lambda match: '{}{}"'.format(match.group(1), new_version),
                content,
            )
        )
        f.truncate()


def prepare_release():
    POETRY_DUMP_VERSION_OUTPUT = re.compile(
        r"Bumping version from \d+\.\d+\.\d+ to (?P<version>\d+\.\d+\.\d+)"
    )

    type_, release_changelog = get_release_info()

    output = run_process(["poetry", "version", type_])
    version_match = POETRY_DUMP_VERSION_OUTPUT.match(output)

    if not version_match:
        print("Unable to bump the project version using Poetry")
        sys.exit(1)

    new_version = version_match.group("version")

    if VERSION_STRINGS:
        for version_file in VERSION_STRINGS:
            file_path = ROOT / version_file
            update_version_strings(file_path, new_version)

    current_date = datetime.utcnow().strftime("%Y-%m-%d")

    old_changelog_data = ""
    header = ""

    with open(CHANGELOG_FILE, "r") as f:
        lines = f.readlines()

    for index, line in enumerate(lines):
        if CHANGELOG_HEADER != line.strip():
            continue

        old_changelog_data = lines[index + 1 :]
        header = lines[: index + 1]
        break

    with open(CHANGELOG_FILE, "w") as f:
        f.write("".join(header))

        new_version_header = f"{new_version} - {current_date}"

        f.write(f"\n{new_version_header}\n")
        f.write(f"{VERSION_HEADER * len(new_version_header)}\n\n")

        f.write(release_changelog)
        f.write("\n")

        f.write("".join(old_changelog_data))



import os
import sys

sys.path.append(os.path.dirname(__file__))  # noqa

from github_release import gh_release_create

from base import (
    run_process,
    check_exit_code,
    get_project_version,
    configure_git,
    PROJECT_NAME,
    REPO_SLUG,
    get_release_info,
)


def create_github_release():
    configure_git()
    version = get_project_version()

    if not version:
        print("Unable to determine the current version")
        sys.exit(1)

    tag_exists = (
        check_exit_code(
            [f'git show-ref --tags --quiet --verify -- "refs/tags/{version}"']
        )
        == 0
    )

    if not tag_exists:
        run_process(["git", "tag", version])
        run_process(["git", "push", "--tags"])

    _, changelog = get_release_info()

    gh_release_create(
        REPO_SLUG,
        version,
        publish=True,
        name=f"{PROJECT_NAME} {version}",
        body=changelog,
        asset_pattern="dist/*",
    )



import os
import sys

sys.path.append(os.path.dirname(__file__))  # noqa

from base import run_process
from create_github_release import create_github_release
from prepare_release import prepare_release
from commit_release import git_commit_and_push


def deploy_release():
    prepare_release()
    run_process(["poetry", "build"])
    create_github_release()
    git_commit_and_push()
    run_process(
        ["poetry", "publish", "-u", "$PYPI_USERNAME", "-p", "$PYPI_PASSWORD"]
    )



import os
import re
import subprocess
import sys

from pathlib import Path
from tomlkit import parse

# Determine CI/CD environment

if os.environ.get("CIRCLECI"):
    CI_SYSTEM = "circleci"
    CIRCLE_PROJECT_USERNAME = os.environ.get("CIRCLE_PROJECT_USERNAME")
    CIRCLE_PROJECT_REPONAME = os.environ.get("CIRCLE_PROJECT_REPONAME")
    REPO_SLUG = f"{CIRCLE_PROJECT_USERNAME}/{CIRCLE_PROJECT_REPONAME}"
elif os.environ.get("TRAVIS"):
    CI_SYSTEM = "travis"
    REPO_SLUG = os.environ.get("TRAVIS_REPO_SLUG")
else:
    CI_SYSTEM = None

# Project root and file name configuration

PROJECT_ROOT = os.environ.get("PROJECT_ROOT")
PYPROJECT_FILE_NAME = os.environ.get("PYPROJECT_FILE_NAME", "pyproject.toml")

if PROJECT_ROOT:
    ROOT = Path(PROJECT_ROOT)
else:
    ROOT = Path(
        subprocess.check_output(["git", "rev-parse", "--show-toplevel"])
        .decode("ascii")
        .strip()
    )

PYPROJECT_FILE = ROOT / PYPROJECT_FILE_NAME

# Retrieve configuration from pyproject file

if os.path.exists(PYPROJECT_FILE):
    config = parse(open(PYPROJECT_FILE).read())
else:
    print(f"Could not find pyproject file at: {PYPROJECT_FILE}")
    sys.exit(1)

PROJECT_NAME = config.get("tool", {}).get("autopub", {}).get("project-name")
if not PROJECT_NAME:
    PROJECT_NAME = config.get("tool", {}).get("poetry", {}).get("name")
if not PROJECT_NAME:
    print(
        "Could not determine project name. Under the pyproject file's "
        '[tool.autopub] header, add:\nproject-name = "YourProjectName"'
    )
    sys.exit(1)

RELEASE_FILE_NAME = (
    config.get("tool", {}).get("autopub", {}).get("release-file", "RELEASE.md")
)
RELEASE_FILE = ROOT / RELEASE_FILE_NAME

CHANGELOG_FILE_NAME = (
    config.get("tool", {}).get("autopub", {}).get("changelog-file", "CHANGELOG.md")
)
CHANGELOG_FILE = ROOT / CHANGELOG_FILE_NAME

CHANGELOG_HEADER = (
    config.get("tool", {}).get("autopub", {}).get("changelog-header", "=========")
)

VERSION_HEADER = config.get("tool", {}).get("autopub", {}).get("version-header", "-")
VERSION_STRINGS = (
    config.get("tool", {}).get("autopub", {}).get("version-strings", [])
)

# Git configuration

GIT_USERNAME = config.get("tool", {}).get("autopub", {}).get("git-username")
GIT_EMAIL = config.get("tool", {}).get("autopub", {}).get("git-email")
if not GIT_USERNAME or not GIT_EMAIL:
    print("git-username and git-email must be defined in the pyproject file")
    sys.exit(1)


def run_process(popenargs):
    return subprocess.check_output(popenargs).decode("ascii").strip()


def git(popenargs):
    # Do not decode ASCII for commit messages so emoji are preserved
    return subprocess.check_output(["git", *popenargs])


def check_exit_code(popenargs):
    return subprocess.call(popenargs, shell=True)


def get_project_version():
    VERSION_REGEX = re.compile(r"^version\s*=\s*\"(?P<version>\d+\.\d+\.\d+)\"$")

    with open(PYPROJECT_FILE) as f:
        for line in f:
            match = VERSION_REGEX.match(line)

            if match:
                return match.group("version")

    return None


def get_release_info():
    RELEASE_TYPE_REGEX = re.compile(r"^[Rr]elease [Tt]ype: (major|minor|patch)$")

    with open(RELEASE_FILE, "r") as f:
        line = f.readline()
        match = RELEASE_TYPE_REGEX.match(line)

        if not match:
            print(
                "The RELEASE file should start with 'Release type' and "
                "specify one of the following values: major, minor, or patch."
            )
            sys.exit(1)

        type_ = match.group(1)
        changelog = "".join([l for l in f.readlines()]).strip()

    return type_, changelog


def configure_git():
    git(["config", "user.name", GIT_USERNAME])
    git(["config", "user.email", GIT_EMAIL])



import os
import sys

sys.path.append(os.path.dirname(__file__))  # noqa

from base import (
    get_project_version,
    git,
    configure_git,
    PROJECT_NAME,
    PYPROJECT_FILE_NAME,
    CHANGELOG_FILE_NAME,
    RELEASE_FILE_NAME,
    VERSION_STRINGS,
)


def git_commit_and_push():
    configure_git()

    version = get_project_version()

    git(["add", PYPROJECT_FILE_NAME])
    git(["add", CHANGELOG_FILE_NAME])

    if VERSION_STRINGS:
        for version_file in VERSION_STRINGS:
            git(["add", version_file])

    git(["rm", "--cached", RELEASE_FILE_NAME])

    git(["commit", "-m", f"Release {PROJECT_NAME} {version}"])
    git(["push", "origin", "HEAD"])



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['autopub']

package_data = \
{'': ['*']}

install_requires = \
['tomlkit>=0.5.5,<0.6.0']

extras_require = \
{'github': ['githubrelease>=1.5.8,<2.0.0']}

entry_points = \
{'console_scripts': ['autopub = autopub.autopub:main']}

setup_kwargs = {
    'name': 'autopub',
    'version': '0.1.0a9',
    'description': 'Automatic package release upon pull request merge',
    'long_description': '# AutoPub\n\nAutoPub enables project maintainers to release new package versions to PyPI by merging pull requests.\n\n## Environment\n\nAutoPub is intended for use with continuous integration (CI) systems and currently supports CircleCI. Projects used with AutoPub are assumed to be managed via [Poetry][].Support for other CI and build systems is planned and contributions adding such support would be welcome.\n\n## Configuration\n\nAutoPub settings can be configured via the `[tool.autopub]` table in the target project’s `pyproject.toml` file. Required settings include Git username and email address:\n\n```toml\n[tool.autopub]\ngit-username = "Your Name"\ngit-email = "your_email@example.com"\n```\n\n## Release Files\n\nContributors should include a `RELEASE.md` file in their pull requests with two bits of information:\n\n* Release type: major, minor, or patch\n* Description of the changes, to be used as the changelog entry\n\nExample:\n\n    Release type: patch\n\n    Add function to update version strings in multiple files.\n\n## Usage\n\nThe following `autopub` sub-commands can be used as steps in your CI flows:\n\n* `autopub check`: Check whether release file exists.\n* `autopub prepare`: Update version strings and add entry to changelog.\n* `autopub commit`: Add, commit, and push incremented version and changelog changes.\n* `autopub githubrelease`: Create a new release on GitHub.\n\n\n[Poetry]: https://poetry.eustace.io\n',
    'author': 'Justin Mayer',
    'author_email': 'entrop@gmail.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
